@@ -15,7 +15,7 @@
 	  (restore-history #f)      ; function to restore history buffer entries from a file
 	  (helpers ())              ; list of functions displaying help strings 
 	  (run #f)                  ; function that fires up a REPL
-	  (top-level-let (sublet (rootlet) :** #f)) ; environment in which evaluation takes place
+	  (top-level-let (sublet (rootlet))) ; environment in which evaluation takes place
 	  (repl-let                 ; environment for keymap functions to access all the REPL innards (cursor-position etc)
 	   
       (with-let (sublet *libc*)
@@ -28,20 +28,20 @@
 	    (call-with-exit
 	     (lambda (return)
 	       (for-each
-		(lambda (symbol)
-		  (let* ((sym (symbol->string symbol))
+		(lambda (par)
+		  (let* ((sym (symbol->string par))
 			 (sym-len (length sym)))
 		    (when (and (>= sym-len text-len)
 			       (string=? text (substring sym 0 text-len)))
 		      (if match
 			  ;; more than one match, save the longest text that all syms match
-			  (let ((min-len (min (string-length match) sym-len)))
-			    (do ((i text-len (+ i 1)))
-				((or (= i min-len)
-				     (not (char=? (match i) (sym i))))
-				 (if (= min-len text-len)
-				     (return text)
-				     (set! match (substring match 0 i))))))
+			  (do ((min-len (min (string-length match) sym-len))
+			       (i text-len (+ i 1)))
+			      ((or (= i min-len)
+				   (not (char=? (match i) (sym i))))
+			       (if (= min-len text-len)
+				   (return text)
+				   (set! match (substring match 0 i)))))
 			  (set! match sym)))))
 		st)
 	       (or match text)))))
@@ -211,11 +211,8 @@
 				(zero? (system (string-append "command -v " var-name " >/dev/null"))))
 		       (set! unbound-case #t)
 		       (if (procedure? ((rootlet) 'system))
-			   (begin
-			     (set! ((*repl* 'top-level-let) '**) (((rootlet) 'system) cur-line #t))
-			     (display ((*repl* 'top-level-let) '**) *stderr*))
-			   (set! ((*repl* 'top-level-let) '**) (system cur-line)))
-		       (set! (h 'result) (symbol " ")))))))
+			   (set! (h 'result) (((rootlet) 'system) cur-line #f))
+			   (set! (h 'result) #f)))))))
 	    
 	    (define (with-repl-let body)
 	      ;; for multiline edits, we will use *missing-close-paren-hook* rather than try to parse the input ourselves.
@@ -420,6 +417,7 @@
 	    
 	    ;; to enable mouse click coords (in xterm anyway), (format *stderr* "~C[?9h" #\escape)
 	    ;;    disable: (format *stderr* "~C[?9l" #\escape)
+	    ;;    this is X10_MOUSE -- probably better to use SET_ANY_EVENT_MOUSE 1003
 	    ;; while enabled, mouse selection instead sends coords to repl (so it's annoying)
 	    ;;    also it's sticky!  exit repl does not clear this flag so mouse is effectively dead
 	    ;; upon click, we get ESC [ M b x y (b = button info, x/y are coords starting at 32=0 -- 6 chars received)
@@ -902,17 +900,15 @@
 				    ;; get the newline out if the expression does not involve a read error
 				    (let ((form (with-input-from-string cur-line #_read)))    ; not libc's read
 				      (newline *stderr*)
-				      (let ((val (eval form (*repl* 'top-level-let))))
+				      (let ((val ((lambda args
+						    (if (pair? (cdr args))
+							(cons 'values args)
+							(car args)))
+						  (eval form (*repl* 'top-level-let)))))
+					(eval `(define ,(string->symbol (format #f "<~D>" (+ (length histtop) 1))) ',val) (rootlet))
 					(if unbound-case
 					    (set! unbound-case #f)
-					    (begin
-					      (format *stderr* "~S~%" val)
-					      ;; this set! of '** has one odd consequence: if val is a lambda expression
-					      ;;   find_closure in s7 will fallback on the current environment trying to
-					      ;;   find an associated name, and the only thing it finds is '**!  So,
-					      ;;   when we type **, we get back **, which seems perverse.  I suppose
-					      ;;   we could trap the string above, see "**" and change to ~W or something.
-					      (set! ((*repl* 'top-level-let) '**) val))))))))
+					    (format *stderr* "~S~%" val)))))))
 			       
 			       (lambda (type info)
 				 (pop-history)               ; remove last history entry
@@ -920,13 +916,20 @@
 				 (return))))
 			   
 			   (lambda (type info)
-			     (format *stderr* "~A:" (red "error"))
-			     (if (and (pair? info)
-				      (string? (car info)))
-				 (format *stderr* " ~A" (apply format #f info))
-				 (if (not (null? info))
-				     (format *stderr* " ~A" info)))
-			     (newline *stderr*)))
+			     (with-let (unlet)
+			       (format *stderr* "~A:" (red "error"))
+			       (let ((op (*s7* 'print-length)))
+				 (if (< op 32) (set! (*s7* 'print-length) 32))
+				 (if (and (pair? info)
+					  (string? (car info)))
+				     (format *stderr* " ~A" (apply format #f info))
+				     (if (not (null? info))
+					 (format *stderr* " ~A" info)))
+				 (if (< op 32) (set! (*s7* 'print-length) op)))
+			       (with-repl-let
+				(lambda ()
+				  (eval `(define ,(string->symbol (format #f "<~D>" (+ (length histtop) 1))) 'error) (rootlet))))
+			       (newline *stderr*))))
 			 
 			 (push-line (copy cur-line))
 			 (new-prompt))))))
@@ -1078,7 +1081,6 @@
 					    (set! str ""))))
 				       (lambda (type info)
 					 (fgets buf 512 stdin)
-					 ;(format *stderr* "add str: ~S~%" (substring buf 0 (- (strlen buf) 1)))
 					 (set! str (string-append str " " (substring buf 0 (- (strlen buf) 1))))))))
 				 (lambda (type info)
 				   (set! str "")
@@ -1157,10 +1159,10 @@
 						    (unless (char=? (str pos) #\tab)
 						      (set! max-cols (max max-cols (- pos start)))
 						      (set! start pos)))
-						  (if (< (+ old-pos 1) (length str))
-						      (if (zero? old-pos)
-							  (set! cur-line (copy str))
-							  (set! cur-line (string-append cur-line (substring str (+ old-pos 1))))))
+						  (when (< (+ old-pos 1) (length str))
+						    (set! cur-line (if (zero? old-pos)
+								       (copy str)
+								       (string-append cur-line (substring str (+ old-pos 1))))))
 						  
 						  ;; if the line is too long, the cursor gets confused, so try to reformat long strings
 						  ;;    this still messes up sometimes
@@ -1200,7 +1202,7 @@
 					 
 					 (lambda (type info)
 					   (set! chars 0)
-					   (move-cursor prompt-row prompt-col)
+					   ;(move-cursor prompt-row prompt-col)
 					   (format *stderr* "internal error: ")
 					   (apply format *stderr* info)
 					   (format *stderr* "~%line ~A: ~A~%" ((owlet) 'error-line) ((owlet) 'error-code))
@@ -1249,7 +1251,7 @@
 			  (help chr)))
 		      
 		      (lambda (type info)
-			(move-cursor prompt-row prompt-col)
+			;(move-cursor prompt-row prompt-col)
 			(format *stderr* "internal error: ")
 			(apply format *stderr* info)
 			(format *stderr* "~%line ~A: ~A~%" ((owlet) 'error-line) ((owlet) 'error-code))
@@ -1394,7 +1396,7 @@
 	       (make-iterator lt)
 	       (letrec ((iterloop 
 			 (let ((iter (make-iterator lt))
-			       (iterator? #t))
+			       (+iterator+ #t))
 			   (lambda ()
 			     (let ((result (iter)))
 			       (if (and (eof-object? result)
@@ -1509,7 +1511,7 @@
 ;;; --------------------------------------------------------------------------------
 #|
 to work in a particular environment:
-    (set! (*repl* 'top-level-let) (sublet (rootlet) :** #f)) ; or any other like *libc*
+    (set! (*repl* 'top-level-let) (sublet (rootlet))) ; or any other like *libc*
 now (define g 43) puts g in the new top-level-let, so ((rootlet) 'g) -> #<undefined>, and ((*repl* 'top-level-let) 'g) -> 43 (= g in repl of course)
 to start with a fresh top-level, just set top-level-let to (sublet (rootlet)) again.
 
